@@ -5,6 +5,7 @@
 (** GTK cache explorer dialog (for "0install store manage") *)
 
 open Zeroinstall.General
+open Support
 open Support.Common
 open Gtk_common
 
@@ -25,8 +26,8 @@ let rec size_of_item system path =
       | Unix.S_CHR | Unix.S_BLK | Unix.S_FIFO | Unix.S_SOCK -> log_warning "Bad file kind for %s" path; 0L
       | Unix.S_DIR ->
           match system#readdir path with
-          | Success items -> items |> Array.fold_left (fun acc item -> Int64.add acc (size_of_item system (path +/ item))) 0L
-          | Problem ex -> log_warning ~ex "Can't scan %s" path; 0L
+          | Ok items -> items |> Array.fold_left (fun acc item -> Int64.add acc (size_of_item system (path +/ item))) 0L
+          | Error ex -> log_warning ~ex "Can't scan %s" path; 0L
 
 (** Get the size for an implementation. Get the size from the .manifest if possible. *)
 let size_of_impl (system:system) path : Int64.t =
@@ -41,7 +42,7 @@ let size_of_impl (system:system) path : Int64.t =
             let line = input_line stream in
             match line.[0] with
             | 'X' | 'F' ->
-                begin match Str.bounded_split_delim U.re_space line 5 with
+                begin match Str.bounded_split_delim XString.re_space line 5 with
                 | [_type; _hash; _mtime; item_size; _name] -> size := Int64.add !size (Int64.of_string item_size)
                 | _ -> () end
             | _ -> ()
@@ -136,10 +137,10 @@ let show_verification_box config ~parent paths =
                incr n_good
             )
             (function 
-              | Safe_exception (msg, _) ->
+              | Safe_exn.T e ->
                 let space = if !n_bad = 0 then "" else "\n\n" in
                 incr n_bad;
-                report_problem @@ Printf.sprintf "%s%s:\n%s\n" space x msg;
+                report_problem @@ Format.asprintf "%s%s:@.%a@." space x Safe_exn.pp e;
                 Lwt.return ()
               | ex -> Lwt.fail ex
             )
@@ -334,7 +335,7 @@ let open_cache_explorer config =
 
   (* Look through cached feeds for implementation owners *)
   let all_feed_urls = FC.list_all_feeds config in
-  all_feed_urls |> StringSet.iter (fun url ->
+  all_feed_urls |> XString.Set.iter (fun url ->
     try
       match FC.get_cached_feed config (`Remote_feed url) with
       | Some feed -> ok_feeds := feed :: !ok_feeds
@@ -347,7 +348,7 @@ let open_cache_explorer config =
   let impl_of_digest = Hashtbl.create 1024 in
   !ok_feeds |> List.iter (fun feed ->
     (* For each implementation... *)
-    feed.F.implementations |> StringMap.iter (fun _id impl ->
+    feed.F.implementations |> XString.Map.iter (fun _id impl ->
       match impl.Impl.impl_type with
       | `Cache_impl info ->
           (* For each digest... *)
@@ -415,8 +416,8 @@ let open_cache_explorer config =
           with Not_found ->
             let extra =
               match config.system#readdir dir with
-              | Problem ex -> ["error:" ^ Printexc.to_string ex]
-              | Success items -> ["files:" ^ (Array.to_list items |> String.concat ",")] in
+              | Error ex -> ["error:" ^ Printexc.to_string ex]
+              | Ok items -> ["files:" ^ (Array.to_list items |> String.concat ",")] in
             ("-", dir, extra, true, true) end
       | paths ->
           (Printf.sprintf "(%d selected items)" (List.length paths), "", [], true, false) in
