@@ -4,6 +4,7 @@
 
 (** The per-component dialog (the one with the Feeds and Versions tabs). *)
 
+open Support
 open Support.Common
 open Gtk_common
 open Zeroinstall.General
@@ -171,9 +172,9 @@ let add_remote_feed ~parent ~watcher ~recalculate tools iface () =
           Lwt.catch
             (fun () ->
                let url = entry#text in
-               if url = "" then raise_safe "Enter a URL";
+               if url = "" then Safe_exn.failf "Enter a URL";
                match Feed_url.parse_non_distro url with
-                 | `Local_feed _ -> raise_safe "Not a remote feed!"
+                 | `Local_feed _ -> Safe_exn.failf "Not a remote feed!"
                  | `Remote_feed _ as feed_url ->
                    let config = tools#config in
                    let fetcher = tools#make_fetcher (watcher :> Progress.watcher) in
@@ -182,9 +183,9 @@ let add_remote_feed ~parent ~watcher ~recalculate tools iface () =
                    recalculate ~force:false
             )
             (function
-              | Safe_exception (msg, _) ->
+              | Safe_exn.T e ->
                 box#misc#set_sensitive true;
-                error_label#set_text msg;
+                error_label#set_text (Safe_exn.msg e);
                 error_label#misc#show ();
                 entry#misc#grab_focus ();
                 Lwt.return ()
@@ -210,14 +211,14 @@ let add_local_feed ~parent ~recalculate config iface () =
     | `DELETE_EVENT | `CANCEL -> box#destroy ()
     | `OK ->
         try
-          let path = box#filename |? lazy (raise_safe "No filename!") in
+          let path = box#filename |? lazy (Safe_exn.failf "No filename!") in
           match Feed_url.parse_non_distro path with
-          | `Remote_feed _ -> raise_safe "Not a local feed!"
+          | `Remote_feed _ -> Safe_exn.failf "Not a local feed!"
           | `Local_feed _ as feed_url ->
               Gui.add_feed config iface feed_url;
               box#destroy ();
               recalculate ~force:false
-        with Safe_exception _ as ex -> Alert_box.report_error ~parent:box ex
+        with Safe_exn.T _ as ex -> Alert_box.report_error ~parent:box ex
   );
 
   box#show ()
@@ -559,13 +560,13 @@ let make_versions_tab config reqs ~recalculate ~watcher window role =
       let (_ready, result) = watcher#results in
       let (selected, impls) = Gui.list_impls result !role in
       let get_overrides =
-        let cache = ref StringMap.empty in
+        let cache = ref XString.Map.empty in
         fun feed ->
-          match !cache |> StringMap.find feed with
+          match !cache |> XString.Map.find_opt feed with
           | Some result -> result
           | None ->
               let result = F.load_feed_overrides config (Feed_url.parse feed) in
-              cache := !cache |> StringMap.add feed result;
+              cache := !cache |> XString.Map.add feed result;
               result in
 
       impls |> List.iter (fun (impl, problem) ->
@@ -573,7 +574,7 @@ let make_versions_tab config reqs ~recalculate ~watcher window role =
         let impl_id = Impl.get_attr_ex FeedAttr.id impl in
         let overrides = get_overrides from_feed in
         let stability_value =
-          match StringMap.find impl_id overrides.F.user_stability with
+          match XString.Map.find_opt impl_id overrides.F.user_stability with
           | Some user_stability -> Stability.to_string user_stability |> String.uppercase_ascii
           | None -> Q.AttrMap.get_no_ns FeedAttr.stability impl.Impl.props.Impl.attrs |> default "testing" in
 
@@ -606,7 +607,7 @@ let make_versions_tab config reqs ~recalculate ~watcher window role =
 
 let create tools ~trust_db reqs role ~recalculate ~select_versions_tab ~watcher =
   let config = tools#config in
-  let title = Printf.sprintf "Properties for %s" (Solver.Model.Role.to_string role) in
+  let title = Format.asprintf "Properties for %a" Solver.Model.Role.pp role in
   let dialog = GWindow.dialog ~title () in
   dialog#set_default_size
     ~width:(-1)
