@@ -7,6 +7,7 @@ Unix.putenv "TZ" "Europe/London"
 open OUnit
 open Zeroinstall
 open Zeroinstall.General
+open Support
 open Support.Common
 open Fake_system
 
@@ -82,9 +83,11 @@ let as_list flags =
 let test_option_parsing () =
   Support.Logging.threshold := Support.Logging.Warning;
 
-  let config, fake_system = get_fake_config None in
+  let config, _fake_system = get_fake_config None in
   let open Options in
 
+  let stdout = Buffer.create 100 in
+
   let p_full raw_args =
     let (raw_options, args, complete) = Support.Argparse.read_args Cli.spec raw_args in
     assert (complete = Support.Argparse.CompleteNothing);
@@ -95,7 +98,7 @@ let test_option_parsing () =
     | Command_tree.Group _ -> assert false
     | Command_tree.Command subcommand ->
     let flags = Support.Argparse.parse_options (Command_tree.options subcommand) raw_options in
-    let options = Cli.get_default_options config in
+    let options = Cli.get_default_options ~stdout:(Format.formatter_of_buffer stdout) config in
     let process = function
       | #common_option as flag -> Common_options.process_common_option options flag
       | _ -> () in
@@ -128,16 +131,19 @@ let test_option_parsing () =
   equal_str_lists ["run"; "foo"] args;
   assert_equal [`Wrapper "gdb"] (as_list flags);
   begin try
-    Support.Argparse.iter_options flags (fun _ -> raise_safe "Error!");
+    Support.Argparse.iter_options flags (fun _ -> Safe_exn.failf "Error!");
     assert false
-  with Safe_exception ("Error!", ctx) ->
-    assert_str_equal "... processing option '-w'" (List.hd !ctx) end;
+  with Safe_exn.T e ->
+    let msg = Format.asprintf "%a" Safe_exn.pp e in
+    assert_str_equal "Error!\n... processing option '-w'" msg end;
 
-  let v = fake_system#collect_output (fun () -> (
+  Buffer.reset stdout;
+  begin
     try ignore @@ p ["-c"; "--version"]; assert false;
     with System_exit 0 -> ()
-  ))
-  in assert (Str.string_match (Str.regexp_string "0install (zero-install)") v 0);
+  end;
+  let v = Buffer.contents stdout in
+  assert (Str.string_match (Str.regexp_string "0install (zero-install)") v 0);
 
   let _, flags, args = p_full ["--version"; "1.2"; "run"; "foo"] in
   equal_str_lists ["run"; "foo"] args;
@@ -154,9 +160,9 @@ let test_option_parsing () =
 let test_run_real tmpdir =
   Unix.putenv "ZEROINSTALL_PORTABLE_BASE" tmpdir;
   let sels_path =
-    if on_windows then "tests\\test_selections_win.xml"
-    else "tests/test_selections.xml" in
-  let argv = [Fake_system.build_dir +/ "0install"; "run"; sels_path] in
+    if on_windows then "data\\test_selections_win.xml"
+    else "data/test_selections.xml" in
+  let argv = [Fake_system.tests_dir +/ "0install"; "run"; sels_path] in
   let line = Support.Utils.check_output real_system input_all argv in
   assert_str_equal "Hello World\n" line
 
@@ -164,11 +170,13 @@ let test_run_real tmpdir =
 let test_run_fake tmpdir =
   let (config, fake_system) = Fake_system.get_fake_config (Some tmpdir) in
   let sels_path = Support.Utils.abspath Fake_system.real_system (
-    if on_windows then "tests\\test_selections_win.xml"
-    else "tests/test_selections.xml"
+    if on_windows then "data\\test_selections_win.xml"
+    else "data/test_selections.xml"
   ) in
   fake_system#add_file sels_path sels_path;
-  try Cli.handle config ["run"; sels_path; "--"; "--arg"]; assert false
+  try
+    Fake_system.check_no_output @@ fun stdout ->
+    Cli.handle ~stdout config ["run"; sels_path; "--"; "--arg"]; assert false
   with Fake_system.Would_exec (search, _env, args) ->
     assert (not search);
     if on_windows then equal_str_lists ["c:\\cygwin\\bin\\env.exe"; "my-prog"; "Hello World"; "--"; "--arg"] args
@@ -258,6 +266,7 @@ let suite =
   Test_gui.suite;
   Test_qdom.suite;
   Test_run.suite;
+  Test_bug_reporter.suite;
  "test_basedir">:: test_basedir;
  "test_portable_base">:: test_portable_base;
  "test_option_parsing">:: (fun () -> collect_logging test_option_parsing);
@@ -270,10 +279,10 @@ let suite =
      assert_str_equal uri (Generic_select.canonical_iface_uri system arg) in
    let check_err arg =
      try (ignore @@ Generic_select.canonical_iface_uri system arg); assert false
-     with Safe_exception _ -> () in
+     with Safe_exn.T _ -> () in
    check "http://example.com/foo.xml" "http://example.com/foo.xml";
-   check "alias:./tests/v1-alias" "http://example.com/alias1.xml";
-   check "alias:./tests/v2-alias" "http://example.com/alias2.xml";
+   check "alias:./data/v1-alias" "http://example.com/alias1.xml";
+   check "alias:./data/v2-alias" "http://example.com/alias2.xml";
    check_err "http://example.com";
  );
  "test_locale">:: (fun () ->
@@ -309,7 +318,7 @@ let show_log_on_failure fn () =
     (* Gc.full_major (); *)
     !async_exception |> if_some (fun ex -> raise ex)
   with ex ->
-    if U.starts_with (Printexc.to_string ex) "OUnitTest.Skip" then ()
+    if XString.starts_with (Printexc.to_string ex) "OUnitTest.Skip" then ()
     else (
       Fake_system.fake_log#dump;
       log_warning ~ex "Test failed";  (* Useful if you want a stack-trace *)
