@@ -4,6 +4,7 @@
 
 open Zeroinstall
 open Zeroinstall.General
+open Support
 open Support.Common
 open OUnit
 module Q = Support.Qdom
@@ -50,6 +51,15 @@ let test_feed = "<?xml version='1.0'?>\n\
   <package-implementation distributions='MacPorts' main='/opt/local/bin/python2.7' package='python27'/>\n\
 </interface>"
 
+let test_duplicates = "<?xml version='1.0'?>\n\
+<interface xmlns='http://zero-install.sourceforge.net/2004/injector/interface' uri='http://repo.roscidus.com/python/python'>\n\
+  <name>Test</name>\n\
+\n\
+  <package-implementation distributions='Arch' package='python2' version='2..!2.7'/>\n\
+  <package-implementation distributions='Arch' main='/usr/bin/python2' package='python2' version='2.7..'/>\n\
+  <package-implementation distributions='Arch' package='python2' version='2..!2.7'/>\n\
+</interface>"
+
 let test_gobject_feed = "<?xml version='1.0'?>\n\
 <interface xmlns='http://zero-install.sourceforge.net/2004/injector/interface' uri='http://repo.roscidus.com/python/python-gobject'>\n\
   <name>gobject</name>\n\
@@ -57,11 +67,11 @@ let test_gobject_feed = "<?xml version='1.0'?>\n\
 </interface>"
 
 let load_feed system xml =
-    let root = Q.parse_input None @@ Xmlm.make_input (`String (0, xml)) |> Element.parse_feed in
+    let root = Q.parse_input (Some "test-input") @@ Xmlm.make_input (`String (0, xml)) |> Element.parse_feed in
     F.parse system root None
 
 let to_impl_list map : _ Impl.t list =
-  StringMap.map_bindings (fun _ impl -> impl) map
+  XString.Map.map_bindings (fun _ impl -> impl) map
 
 let gimp_feed = Test_feed.feed_of_xml Fake_system.real_system "\
   <interface xmlns='http://zero-install.sourceforge.net/2004/injector/interface' uri='http://gimp.org/gimp'>\
@@ -82,7 +92,7 @@ let suite = "distro">::: [
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
 
     let (config, fake_system) = Fake_system.get_fake_config (Some tmpdir) in
-    fake_system#add_file "/var/lib/pacman/local/python2-2.7.2-4/desc" "../tests/arch/local/python2-2.7.2-4/desc";
+    fake_system#add_file "/var/lib/pacman/local/python2-2.7.2-4/desc" (Fake_system.test_data "arch/local/python2-2.7.2-4/desc");
     fake_system#hide_path "/usr/bin/python2";
     fake_system#hide_path "/usr/bin/python3";
     assert (not @@ fake_system#file_exists "/usr/bin/python2");
@@ -95,14 +105,14 @@ let suite = "distro">::: [
     match impls with
     | [impl] ->
         assert_str_equal "2.7.2-4" (Zeroinstall.Version.to_string impl.parsed_version);
-        let run = StringMap.find_safe "run" impl.props.commands in
+        let run = XString.Map.find_safe "run" impl.props.commands in
         assert_str_equal "/bin/python2" (Element.path run.command_qdom |> Fake_system.expect)
     | impls -> assert_failure @@ Printf.sprintf "want 1 Python, got %d" (List.length impls)
   );
 
   "arch2">:: Fake_system.with_fake_config (fun (config, _fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
-    let arch_db = Test_0install.feed_dir +/ "arch" in
+    let arch_db = Fake_system.test_data "arch" in
     let distro = Distro_impls.ArchLinux.arch_distribution ~arch_db ~packagekit config in
 
     distro#get_impls_for_feed ~problem:failwith gimp_feed |> to_impl_list |> assert_equal [];
@@ -114,9 +124,30 @@ let suite = "distro">::: [
     | impls -> assert_failure @@ Printf.sprintf "want 1, got %d" (List.length impls) end;
   );
 
+  "duplicates">:: Fake_system.with_tmpdir (fun tmpdir ->
+    skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
+    let (config, fake_system) = Fake_system.get_fake_config (Some tmpdir) in
+    fake_system#add_file "/var/lib/pacman/local/python2-2.7.2-4/desc" (Fake_system.test_data "arch/local/python2-2.7.2-4/desc");
+    fake_system#hide_path "/usr/bin/python2";
+    fake_system#hide_path "/usr/bin/python3";
+    assert (not @@ fake_system#file_exists "/usr/bin/python2");
+    fake_system#add_dir "/bin" ["python2"; "python3"];
+    let system = (fake_system :> system) in
+    let distro = Distro_impls.ArchLinux.arch_distribution ~packagekit config in
+    let feed = load_feed system test_duplicates in
+    let impls = distro#get_impls_for_feed ~problem:failwith feed |> to_impl_list in
+    let open Impl in
+    match impls with
+    | [impl] ->
+        assert_str_equal "2.7.2-4" (Zeroinstall.Version.to_string impl.parsed_version);
+        let run = XString.Map.find_safe "run" impl.props.commands in
+        assert_str_equal "/bin/python2" (Element.path run.command_qdom |> Fake_system.expect)
+    | impls -> assert_failure @@ Printf.sprintf "want 1 Python, got %d" (List.length impls)
+  );
+
   "slack">:: Fake_system.with_fake_config (fun (config, _fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
-    let slackdir = Test_0install.feed_dir +/ "slack" in
+    let slackdir = Fake_system.test_data "slack" in
     let packages_dir = slackdir +/ "packages" in
     let distro = Distro_impls.Slackware.slack_distribution ~packages_dir ~packagekit config in
 
@@ -132,7 +163,7 @@ let suite = "distro">::: [
 
   "gentoo">:: Fake_system.with_fake_config (fun (config, _fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
-    let pkgdir = Test_0install.feed_dir +/ "gentoo" in
+    let pkgdir = Fake_system.test_data "gentoo" in
     let distro = Distro_impls.Gentoo.gentoo_distribution ~pkgdir ~packagekit config in
 
     distro#get_impls_for_feed ~problem:failwith gimp_feed |> to_impl_list |> assert_equal [];
@@ -165,12 +196,12 @@ let suite = "distro">::: [
 
   "ports">:: Fake_system.with_fake_config (fun (config, _fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
-    let pkg_db = Test_0install.feed_dir +/ "ports" in
+    let pkg_db = Fake_system.test_data "ports" in
     let distro = Distro_impls.Ports.ports_distribution ~pkg_db ~packagekit config in
 
     begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "zeroinstall-injector") |> to_impl_list with
     | [impl] ->
-        assert (U.starts_with (Impl.get_attr_ex "id" impl) "package:ports:zeroinstall-injector:0.41-2:");
+        assert (XString.starts_with (Impl.get_attr_ex "id" impl) "package:ports:zeroinstall-injector:0.41-2:");
         assert_str_equal "0.41-2" @@ Impl.get_attr_ex "version" impl
     | impls -> assert_failure @@ Printf.sprintf "want 1, got %d" (List.length impls) end;
   );
@@ -178,7 +209,7 @@ let suite = "distro">::: [
   "mac-ports">:: Fake_system.with_fake_config (fun (config, fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
     fake_system#set_spawn_handler (Some Fake_system.real_spawn_handler);
-    let pkgdir = Test_0install.feed_dir +/ "macports" in
+    let pkgdir = Fake_system.test_data "macports" in
     let old_path = Unix.getenv "PATH" in
     fake_system#putenv "PATH" (pkgdir ^ ":" ^ old_path);
     Unix.putenv "PATH" (pkgdir ^ ":" ^ old_path);
@@ -209,9 +240,9 @@ let suite = "distro">::: [
     let distro = Distro_impls.generic_distribution ~packagekit config |> Distro.of_provider in
 
     let open Impl in
-    let is_host (id, _impl) = U.starts_with id "package:host:" in
+    let is_host (id, _impl) = XString.starts_with id "package:host:" in
     let find_host impls =
-      try impls |> StringMap.bindings |> List.find is_host |> snd
+      try impls |> XString.Map.bindings |> List.find is_host |> snd
       with Not_found -> assert_failure "No host package found!" in
 
     let root = Q.parse_input None @@ Xmlm.make_input (`String (0, test_feed)) |> Element.parse_feed in
@@ -220,7 +251,7 @@ let suite = "distro">::: [
       let impls = Distro.get_impls_for_feed distro ~problem:failwith feed in
       let host_python = find_host impls in
       let python_run =
-        try StringMap.find_nf "run" host_python.props.commands
+        try XString.Map.find "run" host_python.props.commands
         with Not_found -> assert_failure "No run command for host Python" in
       assert (Fake_system.real_system#file_exists (Element.path python_run.command_qdom |> Fake_system.expect)) in
 
@@ -230,7 +261,7 @@ let suite = "distro">::: [
 
     let impls = Distro.get_impls_for_feed distro ~problem:failwith feed in
     let host_gobject =
-      try impls |> StringMap.bindings |> List.find is_host |> snd
+      try impls |> XString.Map.bindings |> List.find is_host |> snd
       with Not_found -> skip_if true "No host python-gobject found"; assert false in
     let () =
       match host_gobject.props.requires with
@@ -250,7 +281,7 @@ let suite = "distro">::: [
 
   "rpm">:: Fake_system.with_fake_config (fun (config, fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
-    let rpmdir = Test_0install.feed_dir +/ "rpm" in
+    let rpmdir = Fake_system.test_data "rpm" in
     let old_path = Unix.getenv "PATH" in
     Unix.putenv "PATH" (rpmdir ^ ":" ^ old_path);
 
@@ -317,9 +348,9 @@ let suite = "distro">::: [
 
     let feed = F.parse config.system root (Some "/local.xml") in
 
-    assert_equal 0 (StringMap.cardinal feed.F.implementations);
+    assert_equal 0 (XString.Map.cardinal feed.F.implementations);
 
-    let dpkgdir = Test_0install.feed_dir +/ "dpkg" in
+    let dpkgdir = Fake_system.test_data "dpkg" in
     let old_path = Unix.getenv "PATH" in
     Unix.putenv "PATH" (dpkgdir ^ ":" ^ old_path);
     fake_system#putenv "PATH" (dpkgdir ^ ":" ^ old_path);
@@ -349,7 +380,7 @@ let suite = "distro">::: [
         inherit Zeroinstall.Feed_provider_impl.feed_provider config deb
         method! get_feed = function
           | (`Remote_feed "http://example.com/bittorrent") as url ->
-              let result = Some (feed, F.({ last_checked = None; user_stability = StringMap.empty })) in
+              let result = Some (feed, F.({ last_checked = None; user_stability = XString.Map.empty })) in
               cache <- Zeroinstall.Feed_url.FeedMap.add url result cache;
               result
           | _ -> assert false
@@ -365,7 +396,7 @@ let suite = "distro">::: [
 
     (* Part II *)
     let gimp_feed = get_feed "<package-implementation package='gimp'/>" in
-    Distro.get_impls_for_feed deb ~problem:failwith gimp_feed |> assert_equal StringMap.empty;
+    Distro.get_impls_for_feed deb ~problem:failwith gimp_feed |> assert_equal XString.Map.empty;
 
     (* Initially, we only get information about the installed version... *)
     let bt_feed = get_feed "<package-implementation package='python-bittorrent'>\n\
@@ -457,11 +488,11 @@ let suite = "distro">::: [
     feed_provider#add_iface master_feed;
     feed_provider#add_iface imported_feed;
     let scope_filter = { Scope_filter.
-      extra_restrictions = StringMap.empty;
-      os_ranks = Arch.custom_os_ranking StringMap.empty;
-      machine_ranks = Arch.custom_machine_ranking StringMap.empty;
+      extra_restrictions = XString.Map.empty;
+      os_ranks = Arch.custom_os_ranking XString.Map.empty;
+      machine_ranks = Arch.custom_machine_ranking XString.Map.empty;
       languages = Support.Locale.LangMap.empty;
-      allowed_uses = StringSet.empty;
+      allowed_uses = XString.Set.empty;
       may_compile = false;
     } in
     let impl_provider = new Impl_provider.default_impl_provider config (feed_provider :> Feed_provider.feed_provider) scope_filter in
