@@ -6,6 +6,7 @@
 
 open Zeroinstall
 open Zeroinstall.General
+open Support
 open Support.Common
 open OUnit
 
@@ -33,8 +34,8 @@ let parse_sels xml =
     |> Xmlm.make_input
     |> Q.parse_input None
     |> Zeroinstall.Selections.create
-  with Safe_exception _ as ex ->
-    reraise_with_context ex "... parsing %s" xml
+  with Safe_exn.T _ as ex ->
+    Safe_exn.reraise_with ex "... parsing %s" xml
 
 let get_sel_path config sel =
   match Zeroinstall.Selections.get_source sel with
@@ -46,7 +47,7 @@ let remove_cached config selections_path =
   let sels = Zeroinstall.Selections.load_selections config.system selections_path in
   let sel = Zeroinstall.Selections.(get_selected_ex {iface = "http://example.com:8000/Hello.xml"; source = false}) sels in
   let stored = expect @@ get_sel_path config sel in
-  assert (U.starts_with (Filename.basename stored) "sha1");
+  assert (XString.starts_with (Filename.basename stored) "sha1");
   U.rmtree ~even_if_locked:true config.system stored
 
 let fake_gui =
@@ -68,7 +69,7 @@ let install_interceptor system checked_for_gui =
   Update.notify := (fun ~msg ~timeout:_ -> log_info "NOTIFY: 0install: %s" msg)
 
 let do_recipe config fake_system server ?(expected=[[("HelloWorld.tar.bz2", `Serve)]]) name =
-  let feed = Test_0install.feed_dir +/ name in
+  let feed = Fake_system.test_data name in
   server#expect expected;
   let out = run_0install fake_system ["download"; feed; "--command="; "--xml"] in
   let sels = `String (0, out) |> Xmlm.make_input |> Q.parse_input None |> Zeroinstall.Selections.create in
@@ -147,7 +148,7 @@ let suite = "download">::: [
        ("dummy_1-1_all.deb", `Serve)];
     ];
     Fake_system.assert_raises_safe ".*HelloWorld/Missing' does not exist" (lazy (
-      ignore @@ run_0install fake_system ["run"; Test_0install.feed_dir +/ "Recipe.xml"]
+      ignore @@ run_0install fake_system ["run"; Fake_system.test_data "Recipe.xml"]
     ))
   );
 
@@ -247,7 +248,7 @@ let suite = "download">::: [
        \\[dry-run] would store implementation as .*/cache/implementations/sha1=3ce644dc725f1d21cfcf02562c76f375944b266a\n\
        \\[dry-run] would execute: .*HelloWorld/main Hello\n" in
     if not (Str.string_match (Str.regexp expected) out 0) then
-      raise_safe "No match. Got:\n%s" out;
+      Safe_exn.failf "No match. Got:\n%s" out;
   );
 
   "import">:: Server.with_server (fun (config, fake_system) server ->
@@ -266,17 +267,17 @@ let suite = "download">::: [
 
     let domain = "localhost:8000" in
     assert (not (trust_db#is_trusted ~domain "DE937DD411906ACF7C263B396FCF121BE2390E0B"));
-    let out = run_0install fake_system ~stdin:"Y\n" ~include_stderr:true ["import"; "-v"; Test_0install.feed_dir +/ "Hello"] in
+    let out = run_0install fake_system ~stdin:"Y\n" ~include_stderr:true ["import"; "-v"; Fake_system.test_data "Hello"] in
     assert_contains "Warning: Nothing known about this key!" out;
     Fake_system.fake_log#assert_contains "Trusting DE937DD411906ACF7C263B396FCF121BE2390E0B for localhost:8000";
     assert (trust_db#is_trusted ~domain "DE937DD411906ACF7C263B396FCF121BE2390E0B");
 
     (* Check we imported the interface after trusting the key *)
     let hello = FC.get_cached_feed config (`Remote_feed "http://localhost:8000/Hello") |> Fake_system.expect in
-    assert_equal 1 @@ StringMap.cardinal hello.F.implementations;
+    assert_equal 1 @@ XString.Map.cardinal hello.F.implementations;
 
     (* Shouldn't need to prompt the second time *)
-    let out = run_0install fake_system ~stdin:"" ["import"; Test_0install.feed_dir +/ "Hello"] in
+    let out = run_0install fake_system ~stdin:"" ["import"; Fake_system.test_data "Hello"] in
     Fake_system.assert_str_equal "" out;
   );
 
@@ -298,9 +299,9 @@ let suite = "download">::: [
     ));
 
     let feed = Fake_system.expect @@ FC.get_cached_feed config (`Remote_feed native_url) in
-    assert_equal 0 @@ StringMap.cardinal feed.F.implementations;
+    assert_equal 0 @@ XString.Map.cardinal feed.F.implementations;
 
-    let dpkgdir = Test_0install.feed_dir +/ "dpkg" in
+    let dpkgdir = Fake_system.test_data "dpkg" in
     let old_path = Unix.getenv "PATH" in
     Unix.putenv "PATH" (dpkgdir ^ ":" ^ old_path);
     fake_system#putenv "PATH" (dpkgdir ^ ":" ^ old_path);
@@ -310,7 +311,7 @@ let suite = "download">::: [
     Lwt_main.run @@ deb#check_for_candidates ~ui:Fake_system.null_ui feed;
     begin match Test_distro.to_impl_list @@ deb#get_impls_for_feed ~problem:failwith feed with
     | [_impl1; _impl2] -> ()
-    | items -> raise_safe "Got %d!" (List.length items) end;
+    | items -> Safe_exn.failf "Got %d!" (List.length items) end;
 
     Unix.putenv "PATH" old_path;
   );
@@ -331,12 +332,12 @@ let suite = "download">::: [
       [("/0mirror/feeds/http/example.com:8000/Hello.xml/latest.xml", `ServeFile "Hello.xml")];
       [("/0mirror/keys/6FCF121BE2390E0B.gpg", `Serve)];
       [("/HelloWorld.tgz", `Give404)];
-      [("/0mirror/archive/http%3A%23%23example.com%3A8000%23HelloWorld.tgz", `ServeFile "HelloWorld.tgz")];
+      [("/0mirror/archive/http:%23%23example.com:8000%23HelloWorld.tgz", `ServeFile "HelloWorld.tgz")];
     ];
     let out = Fake_system.collect_logging (fun () ->
       run_0install fake_system ["download"; "http://example.com:8000/Hello.xml"; "--xml"]
     ) in
-    Fake_system.fake_log#assert_contains "Primary download failed; trying mirror URL 'http://roscidus.com/0mirror/archive/http%3A%23%23example\\(.\\|%2E\\)com%3A8000%23HelloWorld\\(.\\|%2E\\)tgz'";
+    Fake_system.fake_log#assert_contains {|Primary download failed; trying mirror URL 'http://roscidus.com/0mirror/archive/http\(%3A\|:\)%23%23example\(.\|%2E\)com\(%3A\|:\)8000%23HelloWorld\(.\|%2E\)tgz'|};
     let sels = parse_sels out in
     let sel = Zeroinstall.Selections.get_selected_ex (binary "http://example.com:8000/Hello.xml") sels in
     assert (fake_system#file_exists (expect (get_sel_path config sel) +/ "HelloWorld" +/ "main"))
@@ -357,7 +358,7 @@ let suite = "download">::: [
       [("/Hello.xml", `Serve)];
       [("/6FCF121BE2390E0B.gpg", `Serve)];
       [("/HelloWorld.tgz", `Give404)];
-      [("/0mirror/archive/http%3A%23%23example.com%3A8000%23HelloWorld.tgz", `Give404)];
+      [("/0mirror/archive/http:%23%23example.com:8000%23HelloWorld.tgz", `Give404)];
       [("/0mirror/feeds/http/example.com:8000/Hello.xml/impl/sha1=3ce644dc725f1d21cfcf02562c76f375944b266a",
           `ServeFile "HelloWorld.tar.bz2")];
     ];
@@ -389,7 +390,7 @@ let suite = "download">::: [
       [("/Hello.xml", `Serve)];
       [("/6FCF121BE2390E0B.gpg", `Serve)];
       [("/HelloWorld.tgz", `Give404)];
-      [("/0mirror/archive/http%3A%23%23example.com%3A8000%23HelloWorld.tgz", `Give404)];
+      [("/0mirror/archive/http:%23%23example.com:8000%23HelloWorld.tgz", `Give404)];
       [("/0mirror/feeds/http/example.com:8000/Hello.xml/impl/sha1=3ce644dc725f1d21cfcf02562c76f375944b266a", `Give404)];
     ];
 
@@ -406,7 +407,7 @@ let suite = "download">::: [
       (* The original archive: *)
       ".*http://example.com:8000/HelloWorld.tgz";
       (* Mirror of original archive: *)
-      ".*http://roscidus.com/0mirror/archive/http%3A%23%23example\\(.\\|%2E\\)com%3A8000%23HelloWorld\\(.\\|%2E\\)tgz";
+      {|.*http://roscidus.com/0mirror/archive/http\(:\|%3A\)%23%23example\(.\|%2E\)com\(:\|%3A\)8000%23HelloWorld\(.\|%2E\)tgz|};
       (* Mirror of implementation: *)
       ".*http://roscidus.com/0mirror/feeds/http/example.com:8000/Hello.xml/impl/sha1=3ce644dc725f1d21cfcf02562c76f375944b266a"
     ] |> List.iter Fake_system.fake_log#assert_contains
@@ -419,18 +420,19 @@ let suite = "download">::: [
       Fake_system.collect_logging (fun () ->
         do_recipe config fake_system server ~expected:[
           [("/HelloWorld.tgz", `Give404)];
-          [("/0mirror/archive/http%3A%23%23example.com%3A8000%23HelloWorld.tgz", `ServeFile "HelloWorld.tgz")];
+          [("/0mirror/archive/http:%23%23example.com:8000%23HelloWorld.tgz", `ServeFile "HelloWorld.tgz")];
         ] "Hello.xml"
       ) in
 
-    Fake_system.fake_log#assert_contains "Primary download failed; trying mirror URL \
-    'http://roscidus.com/0mirror/archive/http%3A%23%23example\\(.\\|%2E\\)com%3A8000%23HelloWorld\\(.\\|%2E\\)tgz'...";
+    Fake_system.fake_log#assert_contains @@
+    {|Primary download failed; trying mirror URL |} ^
+    {|'http://roscidus.com/0mirror/archive/http\(:\|%3A\)%23%23example\(.\|%2E\)com\(:\|%3A\)8000%23HelloWorld\(.\|%2E\)tgz'...|};
 
     assert (fake_system#file_exists (path +/ "HelloWorld" +/ "main"))
   );
 
   "selections">:: Server.with_server (fun (config, fake_system) server ->
-    let sels = Zeroinstall.Selections.load_selections config.system (Test_0install.feed_dir +/ "selections.xml") in
+    let sels = Zeroinstall.Selections.load_selections config.system (Fake_system.test_data "selections.xml") in
 
     server#expect [
       [("Hello.xml", `Serve)];
@@ -442,7 +444,7 @@ let suite = "download">::: [
     let sel = Zeroinstall.Selections.get_selected_ex (binary "http://example.com:8000/Hello.xml") sels in
     assert_equal None @@ get_sel_path config sel;
 
-    let out = run_0install fake_system ["download"; Test_0install.feed_dir +/ "selections.xml"] in
+    let out = run_0install fake_system ["download"; Fake_system.test_data "selections.xml"] in
     Fake_system.assert_str_equal "" out;
     Fake_system.fake_log#assert_contains "Automatically approving key for new feed";
 
@@ -483,7 +485,7 @@ let suite = "download">::: [
     let get_mtime path =
       match system#lstat path with
       | Some info -> info.Unix.st_mtime
-      | None -> raise_safe "Missing '%s'" path in
+      | None -> Safe_exn.failf "Missing '%s'" path in
 
     (* Not time for a background update yet *)
     config.freshness <- Some 1000.0;
@@ -695,7 +697,7 @@ let suite = "download">::: [
     let iface = "http://example.com:8000/Hello.xml" in
     let system = config.system in
     let cached = FC.get_save_cache_path config (`Remote_feed iface) in
-    U.copy_file system (Test_0install.feed_dir +/ "Hello-new.xml") cached 0o644;
+    U.copy_file system (Fake_system.test_data "Hello-new.xml") cached 0o644;
 
     let trust_db = new Zeroinstall.Trust.trust_db config in
     let domain = "example.com:8000" in
@@ -727,7 +729,7 @@ let suite = "download">::: [
 
     (* Must finish with the newest version *)
     let actual = U.read_file system cached in
-    let expected = U.read_file system (Test_0install.feed_dir +/ "Hello-new.xml") in
+    let expected = U.read_file system (Fake_system.test_data "Hello-new.xml") in
     assert_equal expected actual
   );
 
@@ -737,7 +739,7 @@ let suite = "download">::: [
     ];
     let tools = Fake_system.make_tools config in
     let feed_provider = new Zeroinstall.Feed_provider_impl.feed_provider config tools#distro in
-    let iface = Test_0install.feed_dir +/ "Binary.xml" in
+    let iface = Fake_system.test_data "Binary.xml" in
     Fake_system.assert_raises_safe "Error downloading 'http://localhost/missing.png': \
                                     The requested URL returned error: 404" (lazy (
       Lwt_main.run @@ Zeroinstall.Gui.download_icon (tools#make_fetcher tools#ui#watcher) feed_provider (`Local_feed iface);
@@ -763,13 +765,13 @@ let suite = "download">::: [
       [("6FCF121BE2390E0B.gpg", `Serve)];
       [("DE937DD411906ACF7C263B396FCF121BE2390E0B", `AcceptKey)];
     ];
-    let out = run_0install fake_system ["select"; Test_0install.feed_dir +/ "selections.xml"] in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "selections.xml"] in
     assert_contains "Version: 1\n" out;
     assert_contains "(not cached)" out;
 
     (* Check background select - update metadata only *)
     let url = "http://example.com:8000/Hello.xml" in
-    F.save_feed_overrides config (`Remote_feed url) F.({last_checked = None; user_stability = StringMap.empty});
+    F.save_feed_overrides config (`Remote_feed url) F.({last_checked = None; user_stability = XString.Map.empty});
     let rel_path = config_site +/ config_prog +/ "last-check-attempt" +/ Zeroinstall.Escape.pretty url in
     let basedirs = Support.Basedir.get_default_config config.system in
     let last_check_attempt = B.load_first config.system rel_path basedirs.B.cache |> expect in
@@ -780,7 +782,7 @@ let suite = "download">::: [
     server#expect [
       [("Hello.xml", `Serve)];
     ];
-    let out = run_0install fake_system ["select"; Test_0install.feed_dir +/ "selections.xml"] in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "selections.xml"] in
     assert_contains "Version: 1\n" out;
     assert_contains "(not cached)" out;
   );
@@ -826,7 +828,7 @@ let suite = "download">::: [
         `Bool false
       ] >>= begin function
       | `WithXML (`List [`String "ok"], xml) -> Lwt.return xml
-      | x -> raise_safe "Bad reply %a" Json_connection.pp_opt_xml x
+      | x -> Safe_exn.failf "Bad reply %a" Json_connection.pp_opt_xml x
       end >>= fun xml ->
       let sels = Selections.create xml in
       let impl = Selections.(get_selected (root_role sels) sels) |> expect in
