@@ -2,6 +2,7 @@
  * See the README file for details, or visit http://0install.net.
  *)
 
+open Support
 open Support.Common
 open Zeroinstall
 open Zeroinstall.General
@@ -22,10 +23,10 @@ let impl_template = Impl.({
     attrs = Q.AttrMap.empty
       |> Q.AttrMap.add_no_ns "id" "test1"
       |> Q.AttrMap.add_no_ns "version" "1.0"
-      |> Q.AttrMap.add_no_ns "from-feed" (Test_0install.feed_dir +/ "test.xml");
+      |> Q.AttrMap.add_no_ns "from-feed" (Fake_system.test_data "test.xml");
     requires = [];
     bindings = [];
-    commands = StringMap.empty;
+    commands = XString.Map.empty;
   };
   stability = Stability.Testing;
   os = None;
@@ -113,7 +114,7 @@ let suite = "fetch">::: [
     Fake_system.assert_raises_safe "Relative URL 'mylib-1.0.zip' in non-local feed 'http://example.com/feed.xml'" @@
       lazy (try_with ~template:remote_impl "<archive href='mylib-1.0.zip' size='100'/>");
 
-    Fake_system.assert_error_contains "tests/mylib-1.0.jar' does not exist"
+    Fake_system.assert_error_contains "data/mylib-1.0.jar' does not exist"
       (fun () -> try_with "<file href='mylib-1.0.jar' dest='lib/mylib.jar' size='100'/>");
 
     Fake_system.assert_error_contains "feed says 100, but actually 321 bytes"
@@ -145,10 +146,11 @@ let suite = "fetch">::: [
       try
         try_with ?digest @@ "<recipe><archive href='recipe-base.tgz' extract='recipe' size='305'/>" ^ xml ^ "</recipe>";
         assert (error = None);
-      with Safe_exception (msg, _) ->
-        let error = error |? lazy (raise_safe "Unexpected error '%s'" msg) in
+      with Safe_exn.T e ->
+        let msg = Safe_exn.msg e in
+        let error = error |? lazy (Safe_exn.failf "Unexpected error '%s'" msg) in
         if not (Str.string_match (Str.regexp error) msg 0) then (
-          raise_safe "Expected error '%s' but got '%s'" error msg
+          Safe_exn.failf "Expected error '%s' but got '%s'" error msg
         ) in
 
     try_recipe ~digest:("sha1new", "d025d1e5c68d349f8106002e821968a5832ff008") "<rename source='rootfile' dest='somefile'/>";
@@ -164,6 +166,11 @@ let suite = "fetch">::: [
 
     try_recipe ~digest:("sha1new", "266fdd7055606c28b299ddc77902b81d500ce946") "<remove path='rootfile'/>";
     try_recipe ~error:"Illegal path '\\.'" "<remove path='.'/>";
+
+    try_recipe ~digest:("sha256new", "JKORWC7SE5HS2GHWK5EU2PNMXTKUOQOODGT72WNI3CI3FSBL6QYA")
+               "<file href='runnable/go.sh' dest='start.sh' executable='false' size='52'/>";
+    try_recipe ~digest:("sha256new", "DOUCL455ISDBNZ5JITFFDNCTOWHRFH3JMOPU2VSOOXSEZTIAQUUQ")
+               "<file href='runnable/go.sh' dest='start.sh' executable='true' size='52'/>";
   );
 
   "local-archive">:: Fake_system.with_fake_config (fun (config, fake_system) ->
@@ -171,13 +178,13 @@ let suite = "fetch">::: [
     let tools = Fake_system.make_tools config in
     let fetcher = tools#make_fetcher tools#ui#watcher in
 
-    let local_iface = Test_0install.feed_dir +/ "LocalArchive.xml" in
+    let local_iface = Fake_system.test_data "LocalArchive.xml" in
     let root = Q.parse_file config.system local_iface |> Zeroinstall.Element.parse_feed in
     let feed = F.parse config.system root (Some local_iface) in
 
     let check ?error ?testfile id =
       try
-        let impl = StringMap.find_safe id feed.F.implementations in
+        let impl = XString.Map.find_safe id feed.F.implementations in
         let digests =
           match impl.Impl.impl_type with
           | `Cache_impl {Impl.digests; _} -> digests
@@ -195,10 +202,11 @@ let suite = "fetch">::: [
             (* Is cached now *)
             let path = Stores.lookup_maybe config.system digests config.stores |? lazy (failwith "missing!") in
             assert (config.system#file_exists (path +/ testfile));
-      with Safe_exception (msg, _) ->
-        let error = error |? lazy (raise_safe "Unexpected error '%s'" msg) in
+      with Safe_exn.T e ->
+        let msg = Safe_exn.msg e in
+        let error = error |? lazy (Safe_exn.failf "Unexpected error '%s'" msg) in
         if not (Str.string_match (Str.regexp error) msg 0) then (
-          raise_safe "Expected error '%s' but got '%s'" error msg
+          Safe_exn.failf "Expected error '%s' but got '%s'" error msg
         ) in
 
     fake_system#set_argv @@ Array.of_list [
@@ -206,14 +214,15 @@ let suite = "fetch">::: [
     ];
     let xml =
       Fake_system.capture_stdout (fun () ->
-        Main.main config.system
+          let stdout = Format.std_formatter in
+          Main.main ~stdout config.system
       ) in
 
     let sels = `String (0, xml) |> Xmlm.make_input |> Q.parse_input None |> Zeroinstall.Selections.create in
     assert (Zeroinstall.Driver.get_unavailable_selections config ~distro:tools#distro sels <> []);
 
-    check ~error:"Local file '.*tests.IDONTEXIST.tgz' does not exist" "impl2";
-    check ~error:"Wrong size for .*.tests.HelloWorld.tgz: feed says 177, but actually 176 bytes" "impl3";
+    check ~error:"Local file '.*data.IDONTEXIST.tgz' does not exist" "impl2";
+    check ~error:"Wrong size for .*.data.HelloWorld.tgz: feed says 177, but actually 176 bytes" "impl3";
     check ~testfile:"HelloWorld" "impl1";
     check ~testfile:"archive.tgz" "impl4";
   );
