@@ -56,12 +56,13 @@ let test ?(package="gnupg") ?(expected_problems=[]) config fake_system =
 
 let suite =
   "packagekit">:: Fake_system.with_fake_config (fun (config, fake_system) ->
+    skip_if (not Zeroinstall.Dbus.have_dbus) "No D-BUS support compiled in";
     let daemon_prog =
       match U.find_in_path config.system "dbus-daemon" with
       | None -> skip_if true "No dbus-daemon"; assert false
       | Some path -> path in
 
-    let dbus_config = Fake_system.tests_dir +/ "dbus.conf" in
+    let dbus_config = Fake_system.test_data "dbus.conf" in
     let addr = "unix:tmpdir=" ^ Fake_system.temp_dir_name in
     let dbus_args = [daemon_prog; "--nofork"; "--print-address"; "--address=" ^ addr; "--config-file"; dbus_config] in
     let r, w = Unix.pipe () in
@@ -81,9 +82,7 @@ let suite =
             assert_equal 0 @@ test config fake_system
               ~expected_problems:["gnupg: PackageKit not available: .*"];
 
-            let destroy =
-              try Lwt_main.run (Pk_service.start [| 0; 8; 1 |])
-              with Safe_exception ("No D-BUS!", _) -> skip_if true "No D-BUS support compiled in"; assert false in
+            let destroy = Lwt_main.run (Pk_service.start [| 0; 8; 1 |]) in
             assert_equal 1 @@ test config fake_system;
             Fake_system.fake_log#assert_contains "confirm: The following components need to be installed using native packages.";
 
