@@ -3,6 +3,7 @@
  *)
 
 open Zeroinstall.General
+open Support
 open Support.Common
 open OUnit
 open Zeroinstall
@@ -13,7 +14,7 @@ module U = Support.Utils
 module StringData =
   struct
     type t = string
-    let to_string s = s
+    let pp = Format.pp_print_string
     let unused = "unused"
   end
 
@@ -46,7 +47,7 @@ let set_of_attrs elem : string list =
 
 let rec fixup_for_windows elem =
   let attrs = elem.Q.attrs |> Q.AttrMap.map (fun value ->
-    if U.starts_with value "/root/" then "c:\\root\\" ^ U.string_tail value 6
+    if XString.starts_with value "/root/" then "c:\\root\\" ^ XString.tail value 6
     else if value = "/root" then "c:\\root"
     else value
   ) in {elem with Q.
@@ -99,11 +100,11 @@ let rec make_all_downloable node =
   )
 
 let linux_multi_scope_filter = { Scope_filter.
-  extra_restrictions = StringMap.empty;
+  extra_restrictions = XString.Map.empty;
   os_ranks = Arch.get_os_ranks Arch.linux;
   machine_ranks = Arch.get_machine_ranks Arch.x86_64 ~multiarch:true;
   languages = Support.Locale.LangMap.empty;
-  allowed_uses = StringSet.empty;
+  allowed_uses = XString.Set.empty;
   may_compile = false;
 }
 
@@ -115,7 +116,7 @@ class fake_feed_provider system (distro:Distro.provider option) =
       try
         let overrides = {
           Feed.last_checked = None;
-          Feed.user_stability = StringMap.empty;
+          Feed.user_stability = XString.Map.empty;
         } in
         Some (Hashtbl.find ifaces url, overrides)
       with Not_found -> None
@@ -126,10 +127,10 @@ class fake_feed_provider system (distro:Distro.provider option) =
     method get_distro_impls feed =
       let overrides = {
         Feed.last_checked = None;
-        Feed.user_stability = StringMap.empty;
+        Feed.user_stability = XString.Map.empty;
       } in
       match distro with
-      | None -> {Feed_provider.impls = StringMap.empty; overrides; problems = []}
+      | None -> {Feed_provider.impls = XString.Map.empty; overrides; problems = []}
       | Some distro ->
           let impls = distro#get_impls_for_feed ~problem:failwith feed in
           {Feed_provider.impls; overrides; problems = []}
@@ -188,7 +189,7 @@ let make_solver_test test_elem =
         feed_provider#add_iface child
     | Some "import-interface" ->
         let leaf = ZI.get_attribute "from-python" child in
-        let root = Support.Qdom.parse_file system @@ Test_0install.feed_dir +/ leaf in
+        let root = Support.Qdom.parse_file system @@ Fake_system.test_data leaf in
         let fix_path attrs =
           if ZI.get_attribute_opt "uri" root = None then (
             attrs |> Q.AttrMap.add_no_ns "local-path" ("./" ^ leaf)
@@ -197,7 +198,7 @@ let make_solver_test test_elem =
     | Some "requirements" ->
         let iface = ZI.get_attribute "interface" child in
         let iface =
-          if U.starts_with iface "./" then U.abspath (fake_system :> system) iface
+          if XString.starts_with iface "./" then U.abspath (fake_system :> system) iface
           else iface in
         reqs := {!reqs with Requirements.
           interface_uri = iface;
@@ -210,7 +211,7 @@ let make_solver_test test_elem =
           let iface = ZI.get_attribute "interface" restricts in
           let expr = ZI.get_attribute "version" restricts in
           reqs := {!reqs with
-            Requirements.extra_restrictions = StringMap.add iface expr !reqs.Requirements.extra_restrictions
+            Requirements.extra_restrictions = XString.Map.add iface expr !reqs.Requirements.extra_restrictions
           }
         );
         fails := ZI.get_attribute_opt "fails" child = Some "true"
@@ -239,7 +240,9 @@ let make_solver_test test_elem =
       (* Selections.as_xml actual_sels |> Q.reindent |> Q.to_utf8 |> print_endline; *)
       assert (ZI.tag (Selections.as_xml actual_sels) = Some "selections");
       if ready then (
-        let changed = Whatchanged.show_changes (fake_system :> system) (Some (Selections.create !expected_selections)) actual_sels in
+        let changed =
+          Fake_system.check_no_output @@ fun stdout ->
+          Whatchanged.show_changes stdout (Some (Selections.create !expected_selections)) actual_sels in
         if changed then
           let sels = Selections.as_xml actual_sels |> Q.reindent |> Q.to_utf8 in
           assert_failure (Printf.sprintf "Ready, but not as expected. Got:\n" ^ sels)
@@ -383,8 +386,8 @@ let suite = "solver">::: [
         val distro_name = "dummy"
         val check_host_python = false
         method! private is_installed = failwith "is_installed"
-        method check_for_candidates = raise_safe "Unexpected check_for_candidates"
-        method! install_distro_packages = raise_safe "install_distro_packages"
+        method check_for_candidates = Safe_exn.failf "Unexpected check_for_candidates"
+        method! install_distro_packages = Safe_exn.failf "install_distro_packages"
         method private get_package_impls query =
           let machine = Some Arch.x86_64 in
           self#add_package_implementation
@@ -429,15 +432,15 @@ let suite = "solver">::: [
           match super#get_feed url with
           | None -> None
           | Some (feed, overrides) ->
-              let overrides = {overrides with Feed.user_stability = StringMap.singleton "preferred_by_user" Stability.Preferred} in
+              let overrides = {overrides with Feed.user_stability = XString.Map.singleton "preferred_by_user" Stability.Preferred} in
               Some (feed, overrides)
 
         method! get_distro_impls feed =
           let result = super#get_distro_impls feed in
-          let overrides = {result.Feed_provider.overrides with Feed.user_stability = StringMap.singleton "package:buggy" Stability.Buggy} in
+          let overrides = {result.Feed_provider.overrides with Feed.user_stability = XString.Map.singleton "package:buggy" Stability.Buggy} in
           {result with Feed_provider.overrides}
       end in
-    feed_provider#add_iface (Support.Qdom.parse_file Fake_system.real_system (Fake_system.tests_dir +/ "ranking.xml"));
+    feed_provider#add_iface (Support.Qdom.parse_file Fake_system.real_system (Fake_system.test_data "ranking.xml"));
     config.network_use <- Minimal_network;
 
     let scope_filter = { linux_multi_scope_filter with Scope_filter.
@@ -501,7 +504,7 @@ let suite = "solver">::: [
 
     let impl_provider = make_impl_provider config linux_multi_scope_filter in
 
-    let iface = Test_0install.feed_dir +/ "Ranking.xml" in
+    let iface = Fake_system.test_data "ranking-cpu.xml" in
 
     let test_solve =
       let impls = (impl_provider#get_implementations iface ~source:false).Impl_provider.impls in
@@ -518,7 +521,7 @@ let suite = "solver">::: [
   "details">:: Fake_system.with_tmpdir (fun tmpdir ->
     let (config, _fake_system) = Fake_system.get_fake_config (Some tmpdir) in
     let import name =
-      U.copy_file config.system (Test_0install.feed_dir +/ name) (cache_path_for config @@ "http://foo/" ^ name) 0o644 in
+      U.copy_file config.system (Fake_system.test_data name) (cache_path_for config @@ "http://foo/" ^ name) 0o644 in
     let iface = "http://foo/Binary.xml" in
     import "Binary.xml";
     import "Compiler.xml";
@@ -584,7 +587,7 @@ let suite = "solver">::: [
 
   "command">:: Fake_system.with_tmpdir (fun tmpdir ->
     let (config, _fake_system) = Fake_system.get_fake_config (Some tmpdir) in
-    let r = Requirements.run (Test_0install.feed_dir +/ "command-dep.xml") in
+    let r = Requirements.run (Fake_system.test_data "command-dep.xml") in
     let distro = Fake_distro.make config in
     let feed_provider = new Feed_provider_impl.feed_provider config distro in
     match Solver.solve_for config (feed_provider :> Feed_provider.feed_provider) r with
@@ -605,7 +608,7 @@ let suite = "solver">::: [
     let (config, _fake_system) = Fake_system.get_fake_config (Some tmpdir) in
 
     let import name =
-      U.copy_file config.system (Test_0install.feed_dir +/ name) (cache_path_for config @@ "http://foo/" ^ name) 0o644 in
+      U.copy_file config.system (Fake_system.test_data name) (cache_path_for config @@ "http://foo/" ^ name) 0o644 in
 
     import "MultiArch.xml";
     import "MultiArchLib.xml";
@@ -644,8 +647,8 @@ let suite = "solver">::: [
 
   "restricts">:: Fake_system.with_tmpdir (fun tmpdir ->
     let (config, _fake_system) = Fake_system.get_fake_config (Some tmpdir) in
-    let uri = Test_0install.feed_dir +/ "Conflicts.xml" in
-    let versions = Test_0install.feed_dir +/ "Versions.xml" in
+    let uri = Fake_system.test_data "Conflicts.xml" in
+    let versions = Fake_system.test_data "Versions.xml" in
     let r = Requirements.run uri in
 
     (* Selects 0.2 as the highest version, applying the restriction to versions < 4. *)
@@ -661,12 +664,12 @@ let suite = "solver">::: [
     Fake_system.assert_str_equal "0.2" @@ Element.version (Zeroinstall.Selections.get_selected_ex (binary uri) results);
     Fake_system.assert_str_equal "3" @@ Element.version (Zeroinstall.Selections.get_selected_ex (binary versions) results);
 
-    let extras = StringMap.singleton uri "0.1" in
+    let extras = XString.Map.singleton uri "0.1" in
     let results = do_solve {r with Requirements.extra_restrictions = extras} in
     Fake_system.assert_str_equal "0.1" @@ Element.version (Zeroinstall.Selections.get_selected_ex (binary uri) results);
     assert (Zeroinstall.Selections.get_selected (binary versions) results = None);
 
-    let extras = StringMap.singleton uri "0.3" in
+    let extras = XString.Map.singleton uri "0.3" in
     let r = {r with Requirements.extra_restrictions = extras} in
     assert_equal false (fst @@ Solver.solve_for config feed_provider r);
   );
@@ -685,7 +688,7 @@ let suite = "solver">::: [
       let root_req = { Solver.Model.
         role = {
           Solver.scope = impl_provider;
-          iface = Test_0install.feed_dir +/ "Langs.xml";
+          iface = Fake_system.test_data "Langs.xml";
           source = false
         };
         command = None;
@@ -751,7 +754,7 @@ let suite = "solver">::: [
 
   "solver">:::
     U.handle_exceptions (fun () ->
-      let root = Support.Qdom.parse_file Fake_system.real_system "tests/solves.xml" in
+      let root = Support.Qdom.parse_file Fake_system.real_system (Fake_system.test_data "solves.xml") in
       let root = if on_windows then fixup_for_windows root else root in
       List.map make_solver_test root.Support.Qdom.child_nodes
     ) ()
