@@ -4,7 +4,7 @@
 
 open OUnit
 open Zeroinstall
-open Support.Common
+open Support
 open Fake_system
 
 let pv v =
@@ -14,7 +14,7 @@ let pv v =
 
 let invalid v =
   try ignore @@ pv v; assert false
-  with Safe_exception _ -> () 
+  with Safe_exn.T _ -> () 
 
 let dummy_impl = Impl.({
   qdom = Element.make_impl Support.Qdom.AttrMap.empty;
@@ -24,7 +24,7 @@ let dummy_impl = Impl.({
   props = {
     attrs = Support.Qdom.AttrMap.empty;
     requires = [];
-    commands = StringMap.empty;   (* (not used; we can provide any command) *)
+    commands = XString.Map.empty;   (* (not used; we can provide any command) *)
     bindings = [];
   };
   parsed_version = Version.parse "0";
@@ -120,7 +120,7 @@ let suite = "versions">::: [
     try
       ignore @@ Version.parse_expr "1.1..2";
       assert false
-    with Safe_exception _ -> ()
+    with Safe_exn.T _ -> ()
   );
 
   "ranges2">:: (fun () ->
