@@ -2,6 +2,7 @@
  * See the README file for details, or visit http://0install.net.
  *)
 
+open Support
 open Support.Common
 
 module U = Support.Utils
@@ -66,7 +67,7 @@ and properties = {
   attrs : AttrMap.t;
   requires : dependency list;
   bindings : binding list;
-  commands : command StringMap.t;
+  commands : command XString.Map.t;
 }
 
 and +'a t = {
@@ -106,7 +107,7 @@ let make_command ?source_hint name path : command =
 let make_distribtion_restriction distros =
   object
     method meets_restriction impl =
-      ListLabels.exists (Str.split U.re_space distros) ~f:(fun distro ->
+      ListLabels.exists (Str.split XString.re_space distros) ~f:(fun distro ->
         match distro, impl.impl_type with
         | "0install", `Package_impl _ -> false
         | "0install", `Cache_impl _ -> true
@@ -119,7 +120,7 @@ let make_distribtion_restriction distros =
   end
 
 let get_attr_ex name impl =
-  AttrMap.get_no_ns name impl.props.attrs |? lazy (raise_safe "Missing '%s' attribute for %a" name Element.fmt impl.qdom)
+  AttrMap.get_no_ns name impl.props.attrs |? lazy (Safe_exn.failf "Missing '%s' attribute for %a" name Element.pp impl.qdom)
 
 let parse_version_element elem =
   let before = Element.before elem in
@@ -163,8 +164,8 @@ let make_version_restriction expr =
         method to_string = "version " ^ expr
       end
     )
-  with Safe_exception (ex_msg, _) as ex ->
-    let msg = Printf.sprintf "Can't parse version restriction '%s': %s" expr ex_msg in
+  with Safe_exn.T e as ex ->
+    let msg = Format.asprintf "Can't parse version restriction '%s': %s" expr (Safe_exn.msg e) in
     log_warning ~ex:ex "%s" msg;
     make_impossible_restriction msg
 
@@ -177,25 +178,25 @@ let parse_dep local_dir node =
   let dep = Element.classify_dep node in
   let iface, node =
     let raw_iface = Element.interface node in
-    if U.starts_with raw_iface "." then (
+    if XString.starts_with raw_iface "." then (
       match local_dir with
       | Some dir ->
           let iface = U.normpath @@ dir +/ raw_iface in
           (iface, Element.with_interface iface node)
       | None ->
-          raise_safe "Relative interface URI '%s' in non-local feed" raw_iface
+          Safe_exn.failf "Relative interface URI '%s' in non-local feed" raw_iface
     ) else (
       (raw_iface, node)
     ) in
 
-  let commands = ref StringSet.empty in
+  let commands = ref XString.Set.empty in
 
   let restrictions = Element.restrictions node |> List.map (fun (`Version child) -> parse_version_element child) in
   Element.bindings node |> List.iter (fun child ->
     let binding = Binding.parse_binding child in
     match Binding.get_command binding with
     | None -> ()
-    | Some name -> commands := StringSet.add name !commands
+    | Some name -> commands := XString.Set.add name !commands
   );
 
   let needs_src = Element.source node |> default false in
@@ -206,7 +207,7 @@ let parse_dep local_dir node =
   in
 
   begin match dep with
-  | `Runner r -> commands := StringSet.add (default "run" @@ Element.command r) !commands
+  | `Runner r -> commands := XString.Set.add (default "run" @@ Element.command r) !commands
   | `Requires _ | `Restricts _ -> () end;
 
   let importance =
@@ -224,7 +225,7 @@ let parse_dep local_dir node =
     dep_iface = iface;
     dep_src = needs_src;
     dep_restrictions = restrictions;
-    dep_required_commands = StringSet.elements !commands;
+    dep_required_commands = XString.Set.elements !commands;
     dep_importance = importance;
     dep_use = Element.use node;
     dep_if_os = Element.os node;
@@ -258,16 +259,16 @@ let existing_source = function
   | {impl_type = `Binary_of source; _} -> source
   | {impl_type = #existing; _} as existing -> existing
 
-let get_command_opt command_name impl = StringMap.find command_name impl.props.commands
+let get_command_opt command_name impl = XString.Map.find_opt command_name impl.props.commands
 
 let get_command_ex command_name impl : command =
-  StringMap.find command_name impl.props.commands |? lazy (raise_safe "Command '%s' not found in %a" command_name Element.fmt impl.qdom)
+  XString.Map.find_opt command_name impl.props.commands |? lazy (Safe_exn.failf "Command '%s' not found in %a" command_name Element.pp impl.qdom)
 
 (** The list of languages provided by this implementation. *)
 let get_langs impl =
   let langs =
     match AttrMap.get_no_ns "langs" impl.props.attrs with
-    | Some langs -> Str.split U.re_space langs
+    | Some langs -> Str.split XString.re_space langs
     | None -> ["en"] in
   Support.Utils.filter_map Support.Locale.parse_lang langs
 
@@ -282,4 +283,4 @@ let get_id impl =
   let feed_url = get_attr_ex FeedAttr.from_feed impl in
   Feed_url.({feed = Feed_url.parse feed_url; id = get_attr_ex FeedAttr.id impl})
 
-let fmt f impl = Format.pp_print_string f (Element.show_with_loc impl.qdom)
+let pp f impl = Element.pp f impl.qdom
