@@ -2,13 +2,14 @@
  * See the README file for details, or visit http://0install.net.
  *)
 
+open Support
 open Support.Common
 
 type t = {
   interface_uri : Sigs.iface_uri;
   command : string option;
   source : bool;
-  extra_restrictions : string StringMap.t;  (* iface -> range *)
+  extra_restrictions : string XString.Map.t;  (* iface -> range *)
   os : Arch.os option;
   cpu : Arch.machine option;
   message : string option;
@@ -22,15 +23,15 @@ let run interface_uri = {
   os = None;
   cpu = None;
   message = None;
-  extra_restrictions = StringMap.empty;
+  extra_restrictions = XString.Map.empty;
   may_compile = false;
 }
 
-let parse_extra json : string StringMap.t =
+let parse_extra json : string XString.Map.t =
   let open Yojson.Basic.Util in
   let add map (key, value) =
-    StringMap.add key (to_string value) map in
-  List.fold_left add StringMap.empty (to_assoc json)
+    XString.Map.add key (to_string value) map in
+  List.fold_left add XString.Map.empty (to_assoc json)
 
 let to_string_option_or_empty x =
   let open Yojson.Basic.Util in
@@ -60,17 +61,17 @@ let of_json json =
       | "os" -> r := {!r with os = parse_os value}
       | "cpu" ->  r := {!r with cpu = parse_machine value}
       | "message" -> r := {!r with message = to_string_option value}
-      | _ -> raise_safe "Unknown requirements field '%s'" key
+      | _ -> Safe_exn.failf "Unknown requirements field '%s'" key
     );
 
     if !r.interface_uri = "" then
-      raise_safe "Missing 'interface_uri'";
+      Safe_exn.failf "Missing 'interface_uri'";
 
     (* Update old before/not-before values *)
     let () = 
       let use expr =
-        assert (!r.extra_restrictions = StringMap.empty);
-        r := {!r with extra_restrictions = StringMap.singleton !r.interface_uri expr} in
+        assert (!r.extra_restrictions = XString.Map.empty);
+        r := {!r with extra_restrictions = XString.Map.singleton !r.interface_uri expr} in
       match !not_before, !before with
       | None, None -> ()
       | Some low, None -> use low
@@ -79,7 +80,7 @@ let of_json json =
 
     !r
   with Type_error (msg, _) ->
-    raise_safe "Bad requirements JSON: %s" msg
+    Safe_exn.failf "Bad requirements JSON: %s" msg
 
 let to_json reqs =
   let maybe name = function
@@ -101,10 +102,10 @@ let to_json reqs =
     maybe "message" message;
     maybe_flag ~default:false "source" source;
     maybe_flag ~default:false "may_compile" may_compile;
-    (if StringMap.is_empty extra_restrictions
+    (if XString.Map.is_empty extra_restrictions
       then []
       else [(
         "extra_restrictions",
-        `Assoc (StringMap.map_bindings (fun k v -> (k, `String v)) extra_restrictions)
+        `Assoc (XString.Map.map_bindings (fun k v -> (k, `String v)) extra_restrictions)
       )]);
   ])
