@@ -2,6 +2,7 @@
  * See the README file for details, or visit http://0install.net.
  *)
 
+open Support
 open Support.Common
 
 module U = Support.Utils
@@ -25,9 +26,9 @@ type role = {
 module Role = struct
   type t = role
 
-  let to_string = function
-    | {iface; source = false} -> iface
-    | {iface; source = true} -> iface ^ "#source"
+  let pp f = function
+    | {iface; source = false} -> Format.pp_print_string f iface
+    | {iface; source = true} -> Format.fprintf f "%s#source" iface
 
   (* Sort the interfaces by URI so we have a stable output. *)
   let compare role_a role_b =
@@ -36,10 +37,7 @@ module Role = struct
     | x -> x
 end
 
-module RoleMap = struct
-  include Map.Make(Role)
-  let find key map = try Some (find key map) with Not_found -> None
-end
+module RoleMap = Map.Make(Role)
 
 type t = {
   root : [`Selections] Element.t;
@@ -61,6 +59,8 @@ type dep_info = {
   dep_required_commands : command_name list;
 }
 
+let as_map t = t.index
+
 let re_initial_slash = Str.regexp "^/"
 let re_package = Str.regexp "^package:"
 
@@ -93,24 +93,24 @@ let root_command sels =
   | None | Some "" -> None
   | Some _ as command -> command
 
-let get_selected role sels = RoleMap.find role sels.index
+let get_selected role sels = RoleMap.find_opt role sels.index
 
 let get_selected_ex role sels =
   get_selected role sels
-  |? lazy (raise_safe "Role '%s' not found in selections!" (Role.to_string role))
+  |? lazy (Safe_exn.failf "Role '%a' not found in selections!" Role.pp role)
 
 let root_role sels =
   let iface = root_iface sels in
   let source = Element.source sels.root
     |? lazy (
       (* This is an old (0install < 2.8) selections document, with no source attribute. *)
-      RoleMap.find {iface; source = true} sels.index <> None
+      RoleMap.find_opt {iface; source = true} sels.index <> None
     ) in
   {iface; source}
 
 let root_sel sels =
   let role = root_role sels in
-  get_selected role sels |? lazy (raise_safe "Can't find a selection for the root (%s)!" (Role.to_string role))
+  get_selected role sels |? lazy (Safe_exn.failf "Can't find a selection for the root (%a)!" Role.pp role)
 
 let requirements sels = {role = root_role sels; command = root_command sels}
 
@@ -177,7 +177,7 @@ let collect_bindings t =
 
   t |> iter (fun role node ->
     try process_impl role node
-    with Safe_exception _ as ex -> reraise_with_context ex "... getting bindings from selection %s" (Role.to_string role)
+    with Safe_exn.T _ as ex -> Safe_exn.reraise_with ex "... getting bindings from selection %a" Role.pp role
   );
   List.rev !bindings
 
