@@ -6,6 +6,7 @@
  * This instantiates the [Solver_core] functor with the concrete 0install types. *)
 
 open General
+open Support
 open Support.Common
 module U = Support.Utils
 module Qdom = Support.Qdom
@@ -27,9 +28,9 @@ module CoreModel = struct
      * This allows e.g. using an old version of a compiler to compile the source for the
      * new version (which requires selecting two different versions of the same interface). *)
     type t = role
-    let to_string = function
-      | {iface; source = false; _} -> iface
-      | {iface; source = true; _} -> iface ^ "#source"
+    let pp f = function
+      | {iface; source = false; _} -> Format.pp_print_string f iface
+      | {iface; source = true; _} -> Format.fprintf f "%s#source" iface
 
     (* Sort the interfaces by URI so we have a stable output. *)
     let compare role_a role_b =
@@ -57,9 +58,9 @@ module CoreModel = struct
     command : command_name option;
   }
 
-  let impl_to_string impl = Support.Logging.asprintf "%a-%a" Version.fmt impl.Impl.parsed_version Impl.fmt impl
+  let pp_impl f impl = Format.fprintf f "%a-%a" Version.pp impl.Impl.parsed_version Impl.pp impl
   let id_of_impl impl = Impl.get_attr_ex FeedAttr.id impl
-  let command_to_string command = Element.show_with_loc command.Impl.command_qdom
+  let pp_command f command = Element.pp f command.Impl.command_qdom
   let describe_problem = Impl_provider.describe_problem
 
   let compare_version a b = compare a.Impl.parsed_version b.Impl.parsed_version
@@ -72,7 +73,7 @@ module CoreModel = struct
       ~props:{ Impl.
         attrs = AttrMap.empty;
         requires = [];
-        commands = StringMap.empty;   (* (not used; we can provide any command) *)
+        commands = XString.Map.empty;   (* (not used; we can provide any command) *)
         bindings = [];
       }
       ~version:(Version.parse "0")
@@ -86,7 +87,7 @@ module CoreModel = struct
 
   let get_command impl name =
     if impl == dummy_impl then Some dummy_command
-    else StringMap.find name impl.Impl.props.Impl.commands
+    else XString.Map.find_opt name impl.Impl.props.Impl.commands
 
   let make_deps role zi_deps self_bindings =
     let impl_provider = role.scope in
@@ -217,7 +218,7 @@ module CoreModel = struct
     (candidates.Impl_provider.rejects, candidates.Impl_provider.feed_problems)
 
   let user_restrictions role =
-    StringMap.find role.iface role.scope#extra_restrictions
+    XString.Map.find_opt role.iface role.scope#extra_restrictions
 end
 
 module Core = Solver_core.Make(CoreModel)
@@ -236,7 +237,7 @@ module Model =
     let format_version impl = Version.to_string impl.Impl.parsed_version
 
     let get_selected role result =
-      RoleMap.find role result.selections |> pipe_some (fun selection ->
+      RoleMap.find_opt role result.selections |> pipe_some (fun selection ->
         let impl = Core.(selection.impl) in
         if impl == dummy_impl then None
         else Some impl
@@ -245,7 +246,7 @@ module Model =
     let requirements result = result.root_req
 
     let explain result role =
-      match RoleMap.find role result.selections with
+      match RoleMap.find_opt role result.selections with
       | Some sel -> Core.explain sel.Core.diagnostics
       | None -> "Role not used!"
 
@@ -253,7 +254,7 @@ module Model =
       result.selections |> RoleMap.map (fun sel -> sel.Core.impl)
 
     let selected_commands result role =
-      match RoleMap.find role result.selections with
+      match RoleMap.find_opt role result.selections with
       | Some selection -> Core.(selection.commands)
       | None -> []
   end
@@ -275,7 +276,7 @@ let get_root_requirements config requirements make_impl_provider =
 
   (* This is for old feeds that have use='testing' instead of the newer
     'test' command for giving test-only dependencies. *)
-  let use = if command = Some "test" then StringSet.singleton "testing" else StringSet.empty in
+  let use = if command = Some "test" then XString.Set.singleton "testing" else XString.Set.empty in
 
   let (host_os, host_machine) = Arch.platform config.system in
   let os = default host_os os in
@@ -285,7 +286,7 @@ let get_root_requirements config requirements make_impl_provider =
   let multiarch = os <> Arch.linux || config.system#file_exists "/lib/ld-linux.so.2" in
 
   let scope_filter = { Scope_filter.
-    extra_restrictions = StringMap.map Impl.make_version_restriction extra_restrictions;
+    extra_restrictions = XString.Map.map Impl.make_version_restriction extra_restrictions;
     os_ranks = Arch.get_os_ranks os;
     machine_ranks = Arch.get_machine_ranks ~multiarch machine;
     languages = config.langs;
@@ -308,7 +309,7 @@ let solve_for config feed_provider requirements =
         match do_solve root_req ~closest_match:true with
         | Some result -> (false, result)
         | None -> failwith "No solution, even with closest_match!"
-  with Safe_exception _ as ex -> reraise_with_context ex "... solving for interface %s" requirements.Requirements.interface_uri
+  with Safe_exn.T _ as ex -> Safe_exn.reraise_with ex "... solving for interface %s" requirements.Requirements.interface_uri
 
 (** Create a <selections> document from the result of a solve.
  * The use of Maps ensures that the inputs will be sorted, so we will have a stable output.
